#ifndef ARTERELATIONS_HH
#define ARTERELATIONS_HH
// *********************************************************************
//
// $Source: /afs/desy.de/user/h/hbxxcus/BEE/arte/include/ArteRelations.hh,v $
// $Revision: 1.1.1.23 $
// $Author: cluerep $
// $Date: 2002/01/18 14:16:54 $
//
// Created: 10-21-99,   C. Borgmeier
//
// *********************************************************************

//
// This file is part of the C++ interface for Arte
// 

//
// ********************************************************************
//
//    createRelation
//
// ********************************************************************
//

// This function will be specialized by non-ZEBRA relations.
// It is dummy here.
template<class T>
void arteSetRelation( ArteRelVector<T>&,
                      ArtePointerBase, ArtePointerBase,
                      unsigned int& l, unsigned int& r )
{}


#if defined(USE_GATE) && !defined(__CINT__)
#include <vector>

//
// This macro defines a relation structure.
// An examples can be found in Clue/MC_Track.hh
//
#define defineRelation( Rel, L, e1, R, e2 ) \
inline ArteManRelationPointer<Rel,R> L::* \
findArteManRelationEntry( const Rel& ) { return &L::e1; } \
\
inline ArteInvManRelationPointer<Rel,L> R::* \
findArteInvManRelationEntry( const Rel& ) { return &R::e2; } \
\
template<> \
inline unsigned int& getArteManRelationInt( const Rel& r, \
					    const L& obj ) \
{ return arteReinterpretAsIntReference \
    ( obj.*findArteManRelationEntry( r ) ); } \
\
template<> \
    inline unsigned int& getArteInvManRelationInt( const Rel& r, \
						   const R& obj ) \
{ return arteReinterpretAsIntReference \
    ( obj.*findArteInvManRelationEntry( r ) ); } \
\
template <> \
    inline void arteSetRelation( ArteRelVector<Rel>& relvec, \
				 ArtePointerBase left, \
				 ArtePointerBase right, \
				 unsigned int& leftPoint, \
				 unsigned int& rightPoint ) \
{ \
    std::vector<ArteRelationIterator<R>::ArteManyToManyRelation>* v \
    = static_cast<std::vector<ArteRelationIterator<R> \
    ::ArteManyToManyRelation>*>( relvec.getVec() ); \
  \
  std::vector<ArteRelationIterator<R>::ArteManyToManyRelation> \
 						      ::value_type r; \
  \
  r.selfel    = reinterpret_cast<int&>( left ); \
  r.el.ArtePointerBase::operator=( right ); \
  \
  reinterpret_cast<int&>(r.next)      = leftPoint; \
  reinterpret_cast<int&>(r.othernext) = rightPoint; \
  \
  v->push_back( r ); \
  \
  rightPoint  = v->size(); \
  leftPoint   = v->size(); \
}\

#endif


template<class T1,class T2>
static inline unsigned int&
arteReinterpretAsIntReference( ArteManRelationPointer<T1,T2>& x )
{
  return reinterpret_cast<unsigned int&>( x );
}


template<class T1,class T2>
static inline unsigned int&
arteReinterpretAsIntReference( ArteInvManRelationPointer<T1,T2>& x )
{
  return reinterpret_cast<unsigned int&>( x );
}


template<class T1,class T2>
static inline unsigned int&
arteReinterpretAsIntReference( const ArteManRelationPointer<T1,T2>& x )
{
  return const_cast<unsigned int&>( reinterpret_cast<const unsigned int&>(x));
}


template<class T1,class T2>
static inline unsigned int&
arteReinterpretAsIntReference( const ArteInvManRelationPointer<T1,T2>& x )
{
  return const_cast<unsigned int&>( reinterpret_cast<const unsigned int&>(x));
}


static unsigned int _arteRelationDummy;

template<class Rel,class T>
unsigned int& getArteManRelationInt( const Rel&r, const T& )
{
  return _arteRelationDummy;
}


template<class Rel,class T>
unsigned int& getArteInvManRelationInt( const Rel&r, const T& )
{
  return _arteRelationDummy;
}


//
// InvMan
//

template<class Relation, class T1, class T2>
void createRelation( ArtePointer<T1> el1, ArtePointer<T2> el2,
		     ArteInvManRelationPointer<Relation,T2> T1::* = 0
		     )
{
#ifndef NO_ARTE
  if( ArteTable<T1>::isArteTable() ) {
    mmbrel_(  ArteInvManRelationPointer<Relation,T2>::getLink().getInt(),
	      el1, el2 );
  } else {
#endif
    arteSetRelation( ArteInvManRelationPointer<Relation,T2>::getLink(),
		     el2, el1,
		     getArteManRelationInt( Relation(), *el2 ),
		     getArteInvManRelationInt( Relation(), *el1 )
		     );
#ifndef NO_ARTE
  }
#endif
}


template<class Relation, class T1, class T2>
void createRelation( ArtePointer<T2> el2, ArtePointer<T1> el1,
		     ArteInvManRelationPointer<Relation,T2> T1::* = 0
		     )
{
#ifndef NO_ARTE
  if( ArteTable<T1>::isArteTable() ) {
    mmbrel_(  ArteInvManRelationPointer<Relation,T2>::getLink().getInt(),
	      el1, el2 );
  } else {
#endif
    arteSetRelation( ArteInvManRelationPointer<Relation,T2>::getLink(),
		     el2, el1,
		     getArteManRelationInt( Relation(), *el2 ),
		     getArteInvManRelationInt( Relation(), *el1 )
		     );
#ifndef NO_ARTE
  }
#endif
}


//
// Man
//

template<class Relation, class T1, class T2>
void createRelation(  ArtePointer<T1> el1, ArtePointer<T2> el2,
		      ArteManRelationPointer<Relation,T2> T1::* e
		      )
{
#ifndef NO_ARTE
  if( ArteTable<T1>::isArteTable() ) {
    mmbrel_(  ArteManRelationPointer<Relation,T2>::getLink().getInt(),
	      el2, el1 );
  } else {
#endif
    arteSetRelation( ArteManRelationPointer<Relation,T2>::getLink(),
		     el1, el2,
		     getArteManRelationInt( Relation(), *el1 ),
		     getArteInvManRelationInt( Relation(), *el2 )
		     );
#ifndef NO_ARTE
  }
#endif
}


template<class Relation, class T1, class T2>
void createRelation(  ArtePointer<T2> el2, ArtePointer<T1> el1,
		      ArteManRelationPointer<Relation,T2> T1::* e
		      )
{
#ifndef NO_ARTE
  if( ArteTable<T1>::isArteTable() ) {
    mmbrel_(  ArteManRelationPointer<Relation,T2>::getLink().getInt(),
	      el2, el1 );
  } else {
#endif
    arteSetRelation( ArteManRelationPointer<Relation,T2>::getLink(),
		     el1, el2,
		     getArteManRelationInt( Relation(), *el1 ),
		     getArteInvManRelationInt( Relation(), *el2 )
		     );
#ifndef NO_ARTE
  }
#endif
}


//
// InvOne
//

template<class Relation, class T1, class T2>
void createRelation( ArtePointer<T1> el1, ArtePointer<T2> el2,
		     ArteInvOneRelationPointer<Relation,T2> T1::* = 0
		     )
{
#ifndef NO_ARTE
  mmbrel_(  ArteInvOneRelationPointer<Relation,T2>::getLink().getInt(),
	    el1, el2 );
#else
#endif
}


template<class Relation, class T1, class T2>
void createRelation( ArtePointer<T2> el2, ArtePointer<T1> el1,
		     ArteInvOneRelationPointer<Relation,T2> T1::* = 0
		     )
{
#ifndef NO_ARTE
  mmbrel_(  ArteInvOneRelationPointer<Relation,T2>::getLink().getInt(),
	    el1, el2 );
#else
#endif
}


//
// inlines
//


#if defined(USE_GATE) && defined(USE_ROOT)
inline TBuffer& operator<<( TBuffer& buf, ArtePointerBase ap )
{
  return buf << ap.index();
}
    

inline TBuffer& operator>>( TBuffer& buf, ArtePointerBase& ap )
{
  int idx;
  buf >> idx;
  ap = idx;
  return buf;
}
    
    
template<class T1, class T2>
inline TBuffer& operator<<( TBuffer& buf, ArteRelationPointer<T1,T2> ap )
{
  return buf << ap.base;
}
    

template<class T1, class T2>
inline TBuffer& operator>>( TBuffer& buf, ArteRelationPointer<T1,T2>& ap )
{
  int idx;
  buf >> idx;
  ap.base = idx;
  return buf;
}
#endif /* USE_ROOT */

#ifndef NO_ARTE
//
// Specialization for MTMT
//
inline void createRelation( ArtePointer<MTRA> el1, ArtePointer<MTRA> el2,
                     ArteInvOneRelationPointer<MTMT,MTRA> MTRA::* = 0
                     )
{
  mmbrel_(  ArteInvOneRelationPointer<MTMT,MTRA>::getLink().getInt(),
            el1, el2 );
}


inline void createRelation( ArtePointer<RSEG> el1, ArtePointer<RSEG> el2,
                     ArteManRelationPointer<RSRS,RSEG> RSEG::* = 0
                     )
{
  mmbrel_(  ArteManRelationPointer<RSRS,RSEG>::getLink().getInt(),
            el1, el2 );
}

#endif /* !NO_ARTE */

#endif
