#ifndef ARTE_POINTER_BASE_HH_
#define ARTE_POINTER_BASE_HH_
// *****************************************************************************
//
// source:
//
// type:      source code
//
// created:   10. Jan 1996 by C. Borgmeier
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description:  This file is part of the C++ interface for Arte
//               the HERA-B reconstruction and analysis tool
//
//               This is the definition of the ArtePointerBase class.
//
//               ArtePointerBase is the base class of the ArtePointer
//               template class.
//
// changes:
// 10 Jan 1996 creation by Christoph Borgmeier, Hartwig Albrecht
// 08 Nov 2000 (TG) added operator-=()
// 13 Nov 2000 (TG) added doc++ comments
// 17 Nov 2000 (TG) putted code from PointerBase.icc into class def.
// 15 Mar 2001 (TG) null() member added
//
// *****************************************************************************
#include <iosfwd>
#include "arte/TObject.hh"

#ifdef USE_GATE
class TBuffer;
#endif


/** Base class for ArtePointers.
    For internal use only.

    @author C. Borgmeier, T. Glebe
*/
//==============================================================================
// ArtePointerBase
//==============================================================================
class ArtePointerBase {
#ifdef USE_GATE
  friend inline TBuffer& operator<<( TBuffer&, ArtePointerBase );
  friend inline TBuffer& operator>>( TBuffer&, ArtePointerBase& );
#endif

public:
  /** @name --- Constructors --- */
  ///
  ArtePointerBase() :
    idx(0) {}
  ///
  ArtePointerBase( UShort_t i ) :
    idx( i ) {}
  ///
  ArtePointerBase( const ArtePointerBase& rhs ) :
    idx( rhs.idx ) {}
  ///
  const ArtePointerBase& operator=( const ArtePointerBase& rhs );

  /** @name --- Access methods --- */
  /// return ArteTable row index
  UShort_t index() const;
  /// set ArteTable row index
  void setInt( const UShort_t index );
  /// is this a NULL pointer?
  bool null() const;
  
  /** @name --- Operators --- */
  ///
  bool operator< ( const ArtePointerBase& rhs) const;
  ///
  bool operator> ( const ArtePointerBase& rhs) const;
  ///
  int  operator+ ( const ArtePointerBase& rhs) const;
  ///
  int  operator- ( const ArtePointerBase& rhs) const;
  ///
  bool operator==( const ArtePointerBase& rhs ) const;
  ///
  bool operator!=( const ArtePointerBase& rhs ) const;
  ///
  ArtePointerBase& operator++();
  ///
  ArtePointerBase& operator--();
  ///
  ArtePointerBase  operator++(int);
  ///
  ArtePointerBase  operator--(int);

  /** @name --- Expert functions --- */
  /// used by operator<<()
  std::ostream& print( std::ostream& os ) const;

#if !defined(ARTE_CXX_COMPAT_MODE) || (ARTE_CXX_COMPAT_MODE==1)
public:
#else
protected:
#endif

  /// Return reference to the private data.
  UShort_t& indexReference();
  /// increment row
  void increment();
  /// decrement row
  void decrement();
  ///
  const ArtePointerBase& operator+=( int offset );
  ///
  const ArtePointerBase& operator-=( int offset );
  
private:
  UShort_t idx;

}; // end of class ArtePointerBase


//==============================================================================
// operator <<
//==============================================================================
std::ostream& operator<<( std::ostream& os, const ArtePointerBase& ap );

#include "PointerBase.icc"
#endif
