#!/bin/bash
##########################################################
#
# type:      shell script
#
# created:   05. Dez 2000
#
# author:    Thorsten Glebe
#            HERA-B Collaboration
#            Max-Planck-Institut fuer Kernphysik
#            Saupfercheckweg 1
#            69117 Heidelberg
#            Germany
#            E-mail: T.Glebe@mpi-hd.mpg.de
#
# Description: script to generate ArteTable, ArteTableIterator
#              template specializations
#
##########################################################

CLASSES="Wire EcalHit RecoTrack RecoVertex MC_Track MC_Vertex Topp TrackClones VDS_Track OTR_SpacePoint RICH_Track MuonTrack"
OUTFILE2="include/ClueDeclaration.hh"
OUTFILE3="include/ClueLinkDef.hh"

if [ -f $OUTFILE2 ]; then
    echo "rm $OUTFILE2"
    rm -f $OUTFILE2
fi
if [ -f $OUTFILE3 ]; then
    echo "rm $OUTFILE3"
    rm -f $OUTFILE3
fi


echo "##########################################################"
echo "# generate $OUTFILE2"
echo "# generate $OUTFILE3"
echo "##########################################################"

##########################################################
# Generate header
##########################################################
(
    echo "#ifndef __ARTEDECLAR_HH
#define __ARTEDECLAR_HH
#ifdef  __CINT__
// #########################################################
// This file is automatically generated by
// ClueSpecialize.sh. Do not edit!
// #########################################################
template <class T> class ArteTable;
template <class T> class ArteTableIterator;


std::ostream& operator<<(std::ostream& os, const ArtePointerBase& l);
"
) > $OUTFILE2

(
    echo "#ifdef __CINT__
// #########################################################
// This file is automatically generated by
// ClueSpecialize.sh. Do not edit!
// #########################################################
#pragma link C++ class ArtePointerBase;
#pragma link C++ class ArtePointer*;
#pragma link C++ class ArteTableIterator*;

#pragma link C++ function operator<<( ostream&, const ArtePointerBase& );
"
) > $OUTFILE3


##########################################################
# Generate body
##########################################################
for i in $CLASSES; do
(
    echo "\
class $i;
std::ostream& operator<<(std::ostream& os, const ArteTable<$i>& l);
std::ostream& operator<<(std::ostream& os, const ArteTableIterator<$i>& l);
"
) >> $OUTFILE2

(
    echo "\
#pragma link C++ class ArteTable<$i>-;
#pragma link C++ function operator<<(ostream&, const ArteTable<$i>&);
#pragma link C++ function operator<<(ostream&, const ArteTableIterator<$i>&);
"
) >> $OUTFILE3
done


##########################################################
# Generate trailer
##########################################################
(
    echo "\
#endif // __CINT__
#endif"
) >> $OUTFILE2

(
    echo "#endif"
) >> $OUTFILE3


