// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   20. Jul 1997
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: Particle property description class
//
// changes:
// 20 Jul 1997 creation by C. Borgmeier
// 17 Nov 2000 (TG) modified header files
// 20 Nov 2000 (TG) added namespace CLUE
// 02 Jan 2001 (TG) improved print()
// 11 Jan 2001 (TG) modifed return type of print()
// 28 Feb 2001 (TG) operator==() added
// 05 Mar 2001 (TG) moved anti() to anti_ptr(), added new anti()
// 15 Aug 2001 (TG) added ToppIf definitions from .hh file
//
// ********************************************************************
#include <iostream>
#include <iomanip>

#include "clue/cluearte.hh"
#include "clue/Topp.hh"

#ifdef NAMESPACE
namespace CLUE {
#endif

  //==============================================================================
  // Constructor
  //==============================================================================
  Topp::Topp() {}


  //==============================================================================
  // ToppIf access methods
  //==============================================================================
  const char* const Topp::name()  const { return Name; }
  float             Topp::mass()  const { return Mass; }
  float             Topp::wid()   const { return Wid; }
  float             Topp::trbw()  const { return Trbw; }
  float             Topp::life()  const { return Life; }
  int               Topp::chrg()  const { return Chrg; }
  bool              Topp::valid() const { return true; }
  const ToppIf&     Topp::anti()  const { return *anti_ptr(); }

  //==============================================================================
  // anti_ptr()
  //==============================================================================
  const ArtePointer<Topp> Topp::anti_ptr() const {
    return getPTR<Topp>( Anti );
  }


  //==============================================================================
  // set_anti
  //==============================================================================
  void Topp::set_anti(const ArtePointer<Topp>& topp) {
    Anti = getIDX<Topp>( topp );
  }


  //==============================================================================
  // operator==
  //==============================================================================
  bool Topp::operator==(const ToppIf& rhs) const {
    return (strncmp(Name, rhs.name(), 13) == 0) ? true : false;
  }
  

  //==============================================================================
  // print
  //==============================================================================
  std::ostream& Topp::print( std::ostream& os ) const { 
    os.setf(ios::right,ios::adjustfield);
    os.setf(ios::scientific,ios::floatfield);
    os << name();
    for(unsigned int i = strlen(name()); i<12; ++i)
      os << " ";
    os << "mass: "    << setw(8) << setprecision(3) << mass()
       << ", life: "  << setw(8) << setprecision(3) << life()
       << ", width: " << setw(8) << setprecision(3) << wid()
       << ", chrg: "  << setw(3)                    << chrg()
       << ", colo: "  << setw(2)                    << colo()
       << ", anti: "  << setw(3)                    << anti_ptr();

    return os;
  }

#ifdef USE_ROOT
  //==============================================================================
  // Root::Print
  //==============================================================================
  void Topp::Print(Option_t *) {
    print( std::cout );
    std::cout << endl;
  }

  ClassImp(Topp)
#endif

#ifdef NAMESPACE
} // namespace CLUE
#endif
