// *****************************************************************************
//
// source:
//
// type:      source code
//
// created:   18. Okt 1999
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: Class to describe a track clone
//
// changes:
// 18 Okt 1999 creation by C. Borgmeier
// 15 Jun 2000 (TG) cleanup
// 17 Nov 2000 (TG) modified header files
// 20 Nov 2000 (TG) added namespace CLUE
// 06 Dez 2000 (TG) added code for CLUE relations
// 07 Dez 2000 (TG) copy constructor code added, operator= code added
// 12 Jan 2001 (TG) removed obsolete include files
// 05 Sep 2001 (TG) RelStreamer: StreamVersion added
//
// *****************************************************************************

#include <iostream>
#include "clue/cluearte.hh"      // needed for ClueRelation
#include "clue/RecoTrack.hh"     // needed by defineRelation
#include "clue/TrackClones.hh"

#ifdef NAMESPACE
namespace CLUE {
#endif

  //==============================================================================
  // Constructor
  //==============================================================================
  TrackClones::TrackClones() :
    rtrack(),
    Type( 0 ) {}

  TrackClones::TrackClones(const TrackClones& rhs) :
    rtrack(),
    Type( rhs.Type ) {}

  TrackClones::TrackClones( const RsegC& t ) :
    rtrack(),
    Type( t ) {}

  TrackClones::~TrackClones() {
    // Clue relations cannot unbook!
    //    delete &rtrack;
  }

  const TrackClones& TrackClones::operator=( const TrackClones& rhs ) {
    if(this == &rhs) return *this;

    rtrack = rhs.rtrack;
    Type   = rhs.Type;
    return *this;
  }

#ifdef USE_ROOT
  //==============================================================================
  // RelStreamer
  //==============================================================================
  /// only availabe in ROOT: call Streamer of relation container
  void TrackClones::RelStreamer(TBuffer& R__b, const Version_t& StreamVersion) {
    rTrack().Streamer(R__b);
  }


  //==============================================================================
  // Streamer
  //==============================================================================
  void TrackClones::Streamer(TBuffer &R__b) {
    // Stream an object of class TrackClones.
    
    if (R__b.IsReading()) {
      Version_t R__v = R__b.ReadVersion(); 
      TObject::Streamer(R__b);
      if (R__v < 2) {
	rtrack.Streamer(R__b);
	rtrack.clear(); // clear relations, because they are read at the end
      }
      Type.Streamer(R__b);
    } else {
      R__b.WriteVersion(TrackClones::IsA());
      TObject::Streamer(R__b);
      Type.Streamer(R__b);
    }
  }


  ClassImp(TrackClones);
#endif

#ifdef NAMESPACE
} // namespace CLUE
#endif
