#ifndef CUSTOM_TRACK_HH
#define CUSTOM_TRACK_HH
// **********************************************************************
//
// source:
//
// type:      source code
//
// created:   22. Apr 1999
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: Quick adaption of track segments from different 
//              detector subsystems.
//
// changes:
// 22 Apr 2000 created by C. Borgmeier
// 03 Nov 2000 (TG) removed #include
// 16 Nov 2000 (TG) added artefwd.hh
// 20 Nov 2000 (TG) added namespace CLUE
//
// **********************************************************************
 
#include "clue/RecoSegment.hh"
#include "arte/artefwd.hh"

#ifdef NAMESPACE
namespace CLUE {
#endif
  class abc; // to fake doc++

  /** Track in vertex detector
      @author C. Borgmeier, T. Glebe
  */
  //==============================================================================
  // VDS_Track
  //==============================================================================
  class VDS_Track: public RecoSegment {
  public:
    ///
    VDS_Track() {}
#ifndef NO_ARTE
    ///
    VDS_Track( const ArtePointer<RSEG>& rseg );
#endif
#ifdef USE_ROOT
    ClassDef(VDS_Track,1)
#endif
  };


  /** Track in RICH detector
      @author C. Borgmeier, T. Glebe
  */
  //==============================================================================
  // RICH_Track
  //==============================================================================
  class RICH_Track: public RecoSegment {
  public:
    ///
    RICH_Track() {}
#ifndef NO_ARTE
    ///
    RICH_Track( const ArtePointer<RSEG>& rseg );
#endif
    ///
    float radius;
    ///
    float signalPhotons;
    ///
    float backgndPhotons;
    ///
    float fraction;
#ifdef USE_ROOT
    ClassDef(RICH_Track,1)
#endif
  };


  /** Space point in outer tracker
      @author C. Borgmeier, T. Glebe
  */
  //==============================================================================
  // OTR_SpacePoint
  //==============================================================================
  class OTR_SpacePoint: public RecoSegment {
  public:
    ///
    OTR_SpacePoint() {}
#ifndef NO_ARTE
    ///
    OTR_SpacePoint( const ArtePointer<RSEG>& rseg );
#endif
    ///
    int nHits;
#ifdef USE_ROOT
    ClassDef(OTR_SpacePoint,1)
#endif
  };


  /** Track in MUON detector
      @author C. Borgmeier, T. Glebe
  */
  //==============================================================================
  // MuonTrack
  //==============================================================================
  class MuonTrack: public RecoSegment {
  public:
    ///
    MuonTrack() {}
#ifndef NO_ARTE
    ///
    MuonTrack( const ArtePointer<RSEG>& rseg );
#endif
#ifdef USE_ROOT
    ClassDef(MuonTrack,1)
#endif
  };

#ifdef NAMESPACE
} // end of namespace CLUE
#endif
#endif /* CUSTOM_TRACK_HH */
