#ifndef __RSEGC_HH
#define __RSEGC_HH
// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   12. Dez 2000
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: Class for HERA-B component id's
//
// changes:
// 12 Dec 2000 (TG) creation
// 01 Jun 2001 (TG) slte, sltmu added
// 25 Jul 2001 (TG) clonerm() added
// 16 Aug 2001 (TG) changed return type of print()
// 28 Sep 2001 (TG) exchanged bit20 by clonerm
//
// ********************************************************************

#include <iosfwd>
#include "arte/TObject.hh"


/**
   HERA-B component id
   @author T. Glebe
*/
//==============================================================================
// Rsegc
//==============================================================================
class RsegC: public TObject {
public:
  /** @name --- Constructors --- */
  ///
  RsegC();
  ///
  RsegC(const unsigned int rhs);
  ///
  ~RsegC();
  ///
  RsegC& operator= (const RsegC& rhs);
  
  /** @name --- Access methods --- */
  /// return bit pattern
  unsigned int cmp() const;
  /// bit 0 ($0\textrm{x}1$): Vertex Detector
  bool vxd()   const;
  /// bit 1 ($0\textrm{x}2$): Pattern Tracker
  bool patt()  const;
  /// bit 2 ($0\textrm{x}4$): Magnet Tracker
  bool magt()  const;
  /// bit 3 ($0\textrm{x}8$): Muon Detector
  bool muon()  const;
  /// bit 4 ($0\textrm{x}10$): Ecal
  bool ecal()  const;
  /// bit 5 ($0\textrm{x}20$): Target Constraint
  bool tar()   const;
  /// bit 6 ($0\textrm{x}40$): Trigger Tracker, betw RICH and ECAL
  bool trit()  const;
  /// bit 7 ($0\textrm{x}80$): not assigned yet
  bool bit7()  const;
  /// bit 8 ($0\textrm{x}100$): match SLT trigger tracks
  bool slte()  const;
  /// bit 9 ($0\textrm{x}200$): match SLT trigger tracks
  bool sltmu() const;
  /// bit 10 ($0\textrm{x}400)$: not assigned yet
  bool bit10() const;
  /// bit 11 ($0\textrm{x}800$): not assigned yet
  bool bit11() const;
  /// bit 12 ($0\textrm{x}1000$): not assigned yet
  bool bit12() const;
  /// bit 13 ($0\textrm{x}2000$): not assigned yet
  bool bit13() const;
  /// bit 14 ($0\textrm{x}4000$): not assigned yet
  bool bit14() const;
  /// bit 15 ($0\textrm{x}8000$): not assigned yet
  bool bit15() const;
  /// bit 16 ($0\textrm{x}10000$): not assigned yet
  bool bit16() const;
  /// bit 17 ($0\textrm{x}20000$): not assigned yet
  bool bit17() const;
  /// bit 18 ($0\textrm{x}40000$): Match SVD-PC (marple RSEG only)
  bool bit18() const;
  /// bit 19 ($0\textrm{x}80000$): Unique VDS-PC matches
  bool uniqpc() const;
  /// bit 20 ($0\textrm{x}100000$): Used by Cloneremove.C
  bool clonerm() const;
  /// bit 21 ($0\textrm{x}200000$): not assigned yet
  bool bit21() const;
  /// bit 22 ($0\textrm{x}400000$): not assigned yet
  bool bit22() const;
  /// bit 23 ($0\textrm{x}800000$): not assigned yet
  bool bit23() const;
  /// bit 24 ($0\textrm{x}1000000$): SLT seeded segment
  bool slt()   const;
  /// bit 25 ($0\textrm{x}2000000$): TRD straw device type
  bool trd()   const;
  /// bit 26 ($0\textrm{x}4000000$): Cherenkov Counter device type
  bool rich()  const;
  /// bit 27 ($0\textrm{x}8000000$): HiPT Chamber device type
  bool ptc()   const;
  /// bit 28 ($0\textrm{x}10000000$): MSGC device type
  bool msg()   const;
  /// bit 29 ($0\textrm{x}20000000$): honeycomb wire chamber device type
  bool wch()   const;
  /// bit 30 ($0\textrm{x}40000000$): silicon strip device type
  bool sil()   const;
  /// bit 31 ($0\textrm{x}80000000$): Kill bit, set if segment is deleted
  bool bitsign() const;

  /** @name --- Operators --- */
  ///
  bool operator== (unsigned int rhs) const;
  ///
  bool operator!= (unsigned int rhs) const;
  ///
  const RsegC& operator|= (const unsigned int rhs);
  ///
  const RsegC& operator&= (const unsigned int rhs);

  /** @name --- Expert functions --- */
  ///
  std::ostream& print(std::ostream& os) const;

private:
  UInt_t r_cmp; // the component bit mask

#ifdef USE_ROOT
public:
  ClassDef(RsegC, 1)
#endif
}; // end of class RsegC

//==============================================================================
// operator <<
//==============================================================================
std::ostream& operator<<( std::ostream& os, const RsegC& t );

#ifndef __CINT__
#include "RsegC.icc"
#endif
#endif
