#ifndef TRACK_CLONES_HH
#define TRACK_CLONES_HH
// *****************************************************************************
//
// source:
//
// type:      source code
//
// created:   18. Okt 1999
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: Class to describe a track clone
//
// changes:
// 18 Okt 1999 creation by C. Borgmeier
// 15 Jun 2000 (TG) cleanup
// 17 Nov 2000 (TG) modified header files
// 20 Nov 2000 (TG) added namespace CLUE
// 06 Dez 2000 (TG) replaced ARTE relations by CLUE relations
// 07 Dez 2000 (TG) copy constructor added, operator= added
// 14 Dez 2000 (TG) RelStreamer method added
// 12 Jan 2001 (TG) added non-const rTrack(), moved RelStreamer() code to .C file
// 25 Jul 2001 (TG) made rtrack non-persistent
// 27 Jul 2001 (TG) changed version to 2
// 15 Aug 2001 (TG) moved type() body to .icc file
// 05 Sep 2001 (TG) RelStreamer: StreamVersion added
//
// *****************************************************************************

#include <iosfwd>
#include "arte/TObject.hh"
#include "arte/artefwd.hh"
#include "grelation/ClueRelation.hh"
#include "clue/RsegC.hh"

#ifdef NAMESPACE
namespace CLUE {
#endif

  class RecoTrack;


  /** Class for tracks sharing segments.
      @memo Clone class
      @author C. Borgmeier, T. Glebe
  */
  //==============================================================================
  // TrackClones
  //==============================================================================
  class TrackClones: public TObject {
  public:
    /** @name --- Constructors --- */
    ///
    TrackClones();
    ///
    TrackClones(const TrackClones& rhs);
    ///
    TrackClones( const RsegC& t );
    ///
    ~TrackClones();

    ///
    const TrackClones& operator=( const TrackClones& rhs );

    /** @name --- Access methods --- */
    /// type corresponds to rseg->cmp; type of incompatibility
    RsegC type() const;
    
    /** @name --- CLUE relations --- */
    /// TrackClones $\leftrightarrow$ RecoTrack
    ClueRelationList<RecoTrack> rtrack;   //! not persistent
    ///
    const ClueRelationList<RecoTrack>& rTrack() const;
    ///
    ClueRelationList<RecoTrack>& rTrack();
    ///
    ClueRelationList<RecoTrack>& 
    container(const ArtePointer<RecoTrack>& p);
    
#ifdef USE_ROOT
    /// only availabe in ROOT: call Streamer of relation container
    void RelStreamer(TBuffer& R__b, const Version_t& StreamVersion);
#endif

  private:
    RsegC Type;

#ifdef USE_ROOT
    ClassDef(TrackClones,2)
#endif
  }; // end of class TrackClones


  //==============================================================================
  // operator <<
  //==============================================================================
  inline std::ostream& operator<<( std::ostream& os, const TrackClones& t ) {
    return os << "(Clone:" << t.type() << ")"; 
  }

#ifdef NAMESPACE
} // namespace CLUE
#endif

#ifndef __CINT__
#include "TrackClones.icc"
#endif
#endif
