#!/bin/bash
##########################################################
#
# type:      shell script
#
# created:   05. Dez 2000
#
# author:    Thorsten Glebe
#            HERA-B Collaboration
#            Max-Planck-Institut fuer Kernphysik
#            Saupfercheckweg 1
#            69117 Heidelberg
#            Germany
#            E-mail: T.Glebe@mpi-hd.mpg.de
#
# Description: script to generate GObjArray, GClonesArray
#              template specializations
#
# 14 Aug 2001 (TG) removed TrackBase
#
##########################################################

CLASSES="RecoSegment Wire EcalHit RecoTrack RecoVertex MC_Track
 MC_Vertex Topp TrackClones VDS_Track OTR_SpacePoint RICH_Track MuonTrack"
# PatternMatch
# OUTFILE="include/GateSpecialize.hh"
OUTFILE2="include/GateDeclaration.hh"
OUTFILE3="include/GateLinkDef.hh"

#  if [ -f $OUTFILE ]; then
#      echo "rm $OUTFILE"
#      rm -f $OUTFILE
#  fi
if [ -f $OUTFILE2 ]; then
    echo "rm $OUTFILE2"
    rm -f $OUTFILE2
fi
if [ -f $OUTFILE3 ]; then
    echo "rm $OUTFILE3"
    rm -f $OUTFILE3
fi

echo "##########################################################"
# echo "# generate $OUTFILE"
echo "# generate $OUTFILE2"
echo "# generate $OUTFILE3"
echo "##########################################################"

##########################################################
# Generate header
##########################################################
(
    echo "#ifndef __GATEDECLAR_HH
#define __GATEDECLAR_HH
#ifdef  __CINT__
// #########################################################
// This file is automatically generated by
// GateSpecialize.sh. Do not edit!
// #########################################################
template <class T> class GObjArray;
template <class T> class GClonesArray;
template <class T> class GObjArrayIterator;
template <class T> class ConstGObjArrayIterator;
"
) > $OUTFILE2

(
    echo "//#ifdef __CINT__
// #########################################################
// This file is automatically generated by
// GateSpecialize.sh. Do not edit!
// #########################################################
#pragma link C++ class GClonesArray*;
#pragma link C++ class GObjArrayIterator*;
#pragma link C++ class ConstGObjArrayIterator*;
"
) > $OUTFILE3
#pragma link C++ class GObjArray*;


##########################################################
# Generate body
##########################################################
for i in $CLASSES; do
(
    echo "
class $i;
std::ostream& operator<<(std::ostream& os, const GObjArray<$i*>& l);
std::ostream& operator<<(std::ostream& os, const GClonesArray<$i*>& l);
std::ostream& operator<<(std::ostream& os, const GObjArrayIterator<$i*>& l);
std::ostream& operator<<(std::ostream& os, const ConstGObjArrayIterator<$i*>& l);
"
) >> $OUTFILE2

(
    echo "//#pragma link C++ class GObjArray<$i*>;
#pragma link C++ function operator<<(ostream&, const GObjArray<$i*>&);
#pragma link C++ function operator<<(ostream&, const GObjArrayIterator<$i*>&);
#pragma link C++ function operator<<(ostream&, const ConstGObjArrayIterator<$i*>&);
#pragma link C++ function operator<<(ostream&, const GClonesArray<$i*>&);
"
) >> $OUTFILE3
done

##########################################################
# Generate trailer
##########################################################
(
    echo "#endif // __CINT__
#endif
"
) >> $OUTFILE2

(
    echo "//#endif"
) >> $OUTFILE3
