#!/bin/bash
##########################################################
#
# type:      shell script
#
# created:   05. Jan 2001
#
# author:    Thorsten Glebe
#            HERA-B Collaboration
#            Max-Planck-Institut fuer Kernphysik
#            Saupfercheckweg 1
#            69117 Heidelberg
#            Germany
#            E-mail: T.Glebe@mpi-hd.mpg.de
#
# Description: script to generate Combiner
#              template specializations
#
# 05 Jan 2001 (TG) creation
# 14 Aug 2001 (TG) removed GObjArray<TrackBase*>
# 29 Aug 2001 (TG) replaced SCombiner by SDCombiner
#
##########################################################

CLASSES="ArteTable<RecoTrack> ArteTable<MC_Track>"
# GObjArray<PatternMatch*>
OUTFILE="include/CombinerSpecialize.hh"
OUTFILE2="include/CombinerDeclaration.hh"
OUTFILE3="include/LinkDef.hh"


if [ -f $OUTFILE ]; then
    echo "rm $OUTFILE"
    rm -f $OUTFILE
fi
if [ -f $OUTFILE2 ]; then
    echo "rm $OUTFILE2"
    rm -f $OUTFILE2
fi
if [ -f $OUTFILE3 ]; then
    echo "rm $OUTFILE3"
    rm -f $OUTFILE3
fi

echo "##########################################################"
echo "# generate $OUTFILE"
echo "# generate $OUTFILE2"
echo "# generate $OUTFILE3"
echo "##########################################################"


##########################################################
# Generate header
##########################################################
(
    echo "\
#ifndef __COMBINERSPECIALIZE_HH
#define __COMBINERSPECIALIZE_HH
// #########################################################
// This file is automatically generated by
// CombinerSpecialize.sh. Do not edit!
// #########################################################
#include <list>
#include <vector>

"
) > $OUTFILE

(
    echo "\
#ifndef __COMBINERDECLAR_HH
#define __COMBINERDECLAR_HH
#ifdef  __CINT__
// #########################################################
// This file is automatically generated by
// CombinerSpecialize.sh. Do not edit!
// #########################################################
template <class T> class IteratorPair;
template <class T> class GCombiner;
template <class T> class GCombinerIterator;
template <class T> class SDCombiner;
template <class T> class SDCombinerIterator;
"
) > $OUTFILE2

(
    echo "#ifdef __CINT__
// #########################################################
// This file is automatically generated by
// ClueRelationSpecialize.sh. Do not edit!
// #########################################################

#pragma link C++ class GC_trait*;
#pragma link C++ class IteratorPair*;
#pragma link C++ class GCombiner*;
#pragma link C++ class GCombinerIterator*;
#pragma link C++ class SDCombiner*;
#pragma link C++ class SDCombinerIterator*;
"
) > $OUTFILE3

##########################################################
# Generate body
##########################################################
for i in $CLASSES; do
(
    echo "
//============================================================================
// The GC-Trait
//============================================================================
template<> struct GC_trait<$i > {
  typedef $i                                           Gvalue_type;
  typedef $i*                                          Gpointer;
  typedef $i* const                                    Gconst_pointer;
  typedef $i&                                          Greference;
  typedef const $i&                                    Gconst_reference;
  typedef std::list<Gpointer>                          GCombiner_v;
  typedef std::list<Gpointer>::iterator                GCombiner_it;
  typedef std::list<Gpointer>::const_iterator          GCombiner_cit;
  typedef std::list<Gpointer>::reverse_iterator        GCombiner_rit;
  typedef std::list<Gpointer>::const_reverse_iterator  GCombiner_rcit;

  typedef $i::iterator                         GIterator;
  typedef IteratorPair<GIterator>                      ItPair;
  typedef std::list<ItPair>                            GIterator_v;
  typedef std::list<ItPair>::iterator                  GIterator_it;
  typedef std::list<ItPair>::const_iterator            GIterator_cit;
  typedef std::list<ItPair>::reverse_iterator          GIterator_rit;
  typedef std::list<ItPair>::const_reverse_iterator    GIterator_rcit;

  typedef std::vector<GIterator>                         GCombination;
  typedef std::vector<GIterator>::iterator               GCombination_it;
  typedef std::vector<GIterator>::const_iterator         GCombination_cit;
  typedef std::vector<GIterator>::reverse_iterator       GCombination_rit;
  typedef std::vector<GIterator>::const_reverse_iterator GCombination_rcit;
};
"
) >> $OUTFILE

(
    echo "
class $i;
struct GC_trait<$i >;
bool collision(GCombinerIterator<$i >);
std::ostream& operator<<(std::ostream& os, const IteratorPair<$i::iterator >& l);
std::ostream& operator<<(std::ostream& os, const GCombiner<$i >& l);
std::ostream& operator<<(std::ostream& os, const GCombinerIterator<$i>& l);
std::ostream& operator<<(std::ostream& os, const SDCombiner<$i >& l);
std::ostream& operator<<(std::ostream& os, const SDCombinerIterator<$i >& l);
"
) >> $OUTFILE2

(
    echo "\
#pragma link C++ class IteratorPair<$i::iterator>-!;
#pragma link C++ class GCombiner<$i >-!;
#pragma link C++ class GCombinerIterator<$i >-!;
#pragma link C++ function collision(GCombinerIterator<$i >&);
#pragma link C++ function operator<<(ostream&, const IteratorPair<$i::iterator >&);
#pragma link C++ function operator<<(ostream&, const GCombiner<$i >&);
#pragma link C++ function operator<<(ostream&, const GCombinerIterator<$i>&);
#pragma link C++ class SDCombiner<$i >-!;
#pragma link C++ class SDCombinerIterator<$i >-!;
#pragma link C++ function operator<<(ostream&, const SDCombiner<$i >&);
#pragma link C++ function operator<<(ostream&, const SDCombinerIterator<$i >&);
"
) >> $OUTFILE3
done

##########################################################
# Generate trailer
##########################################################
(
    echo "#endif"
) >> $OUTFILE

(
    echo "#endif // __CINT__
#endif
"
) >> $OUTFILE2

(
    echo "#endif"
) >> $OUTFILE3
