#ifndef __GFACULTY_HH
#define __GFACULTY_HH
// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   07. Feb 2001
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: Faculty computation via template metaprogramming.
//              Example: cout << "5! = " << Faculty<5>::result << endl;
//
// changes:
// 07 Feb 2001 (TG) creation
//
// ********************************************************************

// recursive definition of faculty
template <unsigned int X>
struct Faculty {
  static const unsigned long int result = X * Faculty<X-1>::result;
};

// base case to terminate recursion
template <> 
struct Faculty<1> {
  static const unsigned long int result = 1;
};

// case 0! = 1
template <> 
struct Faculty<0> {
  static const unsigned long int result = 1;
};

// dynamic computation of faculty
static unsigned long int faculty(unsigned long int i) {
  return (i<2) ? 1 : i * faculty(i-1);
}

// compute n!/k! (n>k)
static unsigned long int noverk(unsigned long int n, unsigned long int k) {
  return (n <= k) ? 1 : n * noverk(n-1,k);
}
#endif
