#ifndef __GETOTHER_HH
#define __GETOTHER_HH
#include "gcombiner/SCombiner.hh"


/**
   A Utility for SCombiner.
   getComplement() stores all objects from container \texttt{T}
   which are not present in the given SCombinerIterator in the
   container given as third argument. Thus the complement with
   respect to the objects in the SCombinerIterator can be accessed.

   The container which is searched for matches with objects from
   the SCombinerIterator and the return container have to be of
   the same type as the one used by SCombiner to build the combinations.

   @author T. Glebe, Michaela Agari
*/         



template <class T, unsigned int X>
bool getComplement(const SCombinerIterator<T,X>& sc,
	       T& input, T& output) {

  typedef typename GC_trait<T>::GCombination_cit GCombination_cit;
  typedef typename T::iterator  Titer;

  bool OK = true;

  const Titer             end = input.end();
  const GCombination_cit cend = sc.end();

  for(Titer it = input.begin(); it != end; ++it) {
    for(GCombination_cit cit = sc.begin(); cit != cend; ++cit) {
      if( &*it == &**cit ) OK = false;
    }
    if(OK) output.push_back(*it);
    OK = true;
  }
	
  return true;
}

#endif
