#ifndef GROVER_WITHOUT_ARTE

#include "arte/arte.hh"
#include "grover/grover.hh"

// Part of the Grover package
// Please see the header file grover.hh
// for authors etc.
//
//
// Modifications:
// When?     Who?     What?
// YY-MM-DD  Name     Modification
// -----------------------------------------------------------------------------
// 01-03-13  Max      Added preprocessor directives
// 01-04-26  Michael Bug fix for extrapolating RSEG 
// 01-04-27  Michael call fill_GEDE_array() if needed
//                   this should make it possible to use this propergation
//                   for RTRA's also.
// 01-05-10 Michael  removed factor 2 from coeff for ms estimation.
//

const int    NGEWA = 1500;
extern int  ngewa;
extern float GEWA_xmin[NGEWA], GEWA_xmax[NGEWA];
extern float GEWA_ymin[NGEWA], GEWA_ymax[NGEWA];
extern float GEWA_zmin[NGEWA], GEWA_zmax[NGEWA];
extern float GEWA_ms2[NGEWA];
extern bool fill_GEDE_array_called;
extern int new_geo_for_ms;

extern "C" {
  void fill_GEDE_array();
}

int GroverCatsExtrap(const float z0,
		     float* x, float* y, const float* z, float* ax, float* ay,
		     const float* inversep,
		     float* cov)

{    /****************************************************************
     *                                                              *
     *     This routine provides a procedure for the track          *
     *     parameters extrapolation to a vertex point               *
     *                                                              *
     *     (c) Dmitry Emeliyanov, 2000, DESY HH.                    *
     *     changed coefficient 02-07-00 - wrong MS model  DE        *
     ***************************************************************/
    
  int   gewa[NGEWA];
  int   i, k, g, nfound;
  float Rk[4], Gk[4][4];
  float coeff, pinv, xp, yp, zp, l2sqrt, zprev, ms2, D_z;
  float Ax, Bx, Ay, By;

  if ( ! fill_GEDE_array_called) fill_GEDE_array();

  if ((cov[0]< 0.0)||(cov[2]< 0.0)||(cov[5]< 0.0)||(cov[9]< 0.0)){
    return FAILURE;
  }

  Ax = *ax;   Bx = *x - (*ax)*(*z);
  Ay = *ay;   By = *y - (*ay)*(*z);
    
  l2sqrt = 1.0+Ax*Ax+Ay*Ay; l2sqrt = l2sqrt*sqrt(l2sqrt);
    
  pinv = *inversep;
  if (pinv == 0.0) {
    pinv = 0.1;
    cout << "*** GroverCatsExtrap: WARNING:"
	 << endl << "*** track has no momentum - using 10 GeV!" << endl;
  }

  if (!new_geo_for_ms)
    coeff = 2.0*(1.0+0.105*0.105*pinv*pinv)*pinv*pinv; //muon mass corr.
  else
    coeff =     (1.0+0.105*0.105*pinv*pinv)*pinv*pinv; //muon mass corr.
      
  Rk[0] = *x;   Rk[1] = *y;   Rk[2] = *ax;    Rk[3] = *ay;
      
  Gk[0][0] = cov[0];    Gk[0][1] = cov[1];
  Gk[1][1] = cov[2];    Gk[0][2] = cov[3];
  Gk[1][2] = cov[4];    Gk[2][2] = cov[5];
  Gk[0][3] = cov[6];    Gk[1][3] = cov[7];
  Gk[2][3] = cov[8];    Gk[3][3] = cov[9];
      
  int ngewaf = 0;
  nfound = 0;

  for(i=0; i<ngewa; i++) {
    if (GEWA_zmax[i]<z0) continue;
    if (GEWA_zmax[i]>(*z)) break;
    ngewaf = i;
    zp = 0.5*(GEWA_zmin[i]+GEWA_zmax[i]);
    xp = Ax*zp+Bx;
    yp = Ay*zp+By;
    
    if ((xp>=GEWA_xmin[i])&&(xp<=GEWA_xmax[i])&&
	(yp>=GEWA_ymin[i])&&(yp<=GEWA_ymax[i]))
      gewa[nfound++]=i;
  }


     
  /********************* Fit for the point at z0 *********************/
    
  zprev = *z;
    
  for(k=nfound-1; k>=0; k--) {
    g  = gewa[k];
    ms2= GEWA_ms2[g];
      
    /***** Multiple scattering corrections ****/
      
    Gk[2][2]+=(1.0+Rk[2]*Rk[2])*l2sqrt*coeff*ms2;
    Gk[3][3]+=(1.0+Rk[3]*Rk[3])*l2sqrt*coeff*ms2;
    Gk[2][3]+=Rk[2]*Rk[3]*l2sqrt*coeff*ms2;
      
    /***** Prediction step: from zprev to zmin *****/
    D_z=GEWA_zmin[g]-zprev; 
    zprev=GEWA_zmin[g];
    Rk[0]+=D_z*Rk[2];
    Rk[1]+=D_z*Rk[3];
    Gk[0][0]+=D_z*(2.0*Gk[0][2]+D_z*Gk[2][2]);
    Gk[0][1]+=D_z*(Gk[1][2]+Gk[0][3]+D_z*Gk[2][3]);
    Gk[0][2]+=D_z*Gk[2][2];
    Gk[0][3]+=D_z*Gk[2][3];
    Gk[1][1]+=D_z*(2.0*Gk[1][3]+D_z*Gk[3][3]);
    Gk[1][2]+=D_z*Gk[2][3];
    Gk[1][3]+=D_z*Gk[3][3];
    }
    
  /***** Prediction step : from zprev to z0 *****/
  
  D_z=z0-zprev;
  Rk[0]+=D_z*Rk[2];
  Rk[1]+=D_z*Rk[3];
  Gk[0][0]+=D_z*(2.0*Gk[0][2]+D_z*Gk[2][2]);
  Gk[0][1]+=D_z*(Gk[1][2]+Gk[0][3]+D_z*Gk[2][3]);
  Gk[0][2]+=D_z*Gk[2][2];
  Gk[0][3]+=D_z*Gk[2][3];
  Gk[1][1]+=D_z*(2.0*Gk[1][3]+D_z*Gk[3][3]);
  Gk[1][2]+=D_z*Gk[2][3];
  Gk[1][3]+=D_z*Gk[3][3];

  // Modify the arguments!!!
  *x  = Rk[0];   *y  = Rk[1];
  *ax = Rk[2];   *ay = Rk[3];
  
  cov[0] = Gk[0][0];    cov[1] = Gk[0][1];
  cov[2] = Gk[1][1];    cov[3] = Gk[0][2];
  cov[4] = Gk[1][2];    cov[5] = Gk[2][2];
  cov[6] = Gk[0][3];    cov[7] = Gk[1][3];
  cov[8] = Gk[2][3];    cov[9] = Gk[3][3];


  if ((cov[0]< 0.0)||(cov[2]< 0.0)||(cov[5]< 0.0)||(cov[9]< 0.0)) {
    return FAILURE;
  }

  return SUCCESS;   
}


int GroverCatsExtrap(const ArtePointer<RSEG> rseg, const float z0,
                     float* x, float* y, const float* z, float* ax, float* ay,
                     const float* inversep,
                     float* cov)

{ 
  return  GroverCatsExtrap(z0, x, y, z, ax, ay, inversep, cov);
}

#endif
