#!/bin/bash
##########################################################
#
# type:      shell script
#
# created:   16. Dec 2000
#
# author:    Thorsten Glebe
#            HERA-B Collaboration
#            Max-Planck-Institut fuer Kernphysik
#            Saupfercheckweg 1
#            69117 Heidelberg
#            Germany
#            E-mail: T.Glebe@mpi-hd.mpg.de
#
# Description: script to set up the project directory structure
#
##########################################################

PROG=`basename $0`
BEEPATH=`dirname $0`
HELP="usage: $PROG <project name>"

if [ $# -ne 1 ]; then
    echo $HELP
    exit 1;
else
    PROJ=$1
fi

echo "Create directory structure for BEE project $PROJ..."
mkdir -p $PROJ/include
mkdir -p $PROJ/cintdict
mkdir -p $PROJ/doc

echo "Create template files..."
#
# create Makefile
# 
(
echo "\
##########################################################
# BEE ROOT directory
##########################################################
BEEROOT   = $BEEPATH
HBSCRATCH = .

##########################################################
# unique name of BEE project
##########################################################
project   = $PROJ

##########################################################
# main makefile
##########################################################
include \$(BEEROOT)/Makefile.setup
include \$(BEEROOT)/Makefile.targets

##########################################################
# install: commands executed before compilation
##########################################################
install:

##########################################################
# Turn on rootcint debugging
##########################################################
# ROOTOPT = -t

##########################################################
# give binary other name than a.out
##########################################################
# binary =
"
) > $PROJ/Makefile

#
# create LinkDef.hh
#
(
echo "\
#ifdef __CINT__

#pragma link off all globals;
#pragma link off all classes;
#pragma link off all functions;

#pragma link C++ nestedclasses;
#pragma link C++ nestedtypedefs;

#endif
"
) > $PROJ/cintdict/LinkDef.hh

#
# create doc/Makefile
#
(
echo -e "\
# -*-makefile-*-

DOCXX = doc++

project = $PROJ
texdir=\$(PWD)
wwwdir=\$(PWD)

all: manual.ps \$(project).ps

html:
\t\$(DOCXX) -B banner.txt -d \$(wwwdir) \$(project).dxx
\tchmod a+x \$(wwwdir)/index.html

manual.tex: \$(project).dxx
\t\$(DOCXX) -t -o \$(texdir)/manual.tex\
 -ep a4wide -ep helvetic -ep bookman \$(project).dxx

%.ps: %.tex
\t latex \$(texdir)/$^
\t latex \$(texdir)/$^
\t latex \$(texdir)/$^
\t dvips -o \$(texdir)/\$@ \$(texdir)/\$(addsuffix .dvi, \$(basename \$^))

ps: manual.ps
\t gv \$(texdir)/manual.ps

clean:
\t rm -rf core *.swp *~ *.aux *.dvi *.log *.toc
\t rm -rf manual* *.log *.aux *.toc \$(project).ps

htmlclean:
\t cd \$(wwwdir); rm *.html *.gif dxx* *.class gifs.db .1.html
"
) > $PROJ/doc/Makefile

#
# create doc/proj.dxx
#
(
echo "\
/**@name $PROJ

   @memo <your memo>
.
   @author <your name>
*/
//@{
//@Include:     ../include/<your header files>
//@}
"
) > $PROJ/doc/$PROJ.dxx

#
# create doc/proj.tex
#
(
echo "\
\documentclass{article}
\usepackage{verbatim,color}
\usepackage{graphicx}

\newcommand{\tcb}[1]{\textcolor{blue}{#1}}
\newenvironment{code}[1]{\color{#1}\verbatim}{\endverbatim\color{black}}

\title{}

\author{}

\begin{document}
\maketitle
\abstract{}
\tableofcontents
%
% Section
%
\section{Introduction}
%
% Bibliography
%
\begin{thebibliography}{99}
\end{thebibliography}
\end{document}
"
) > $PROJ/doc/$PROJ.tex
exit 0;