#ifndef __GPATTERNGSELECT_HH
#define __GPATTERNGSELECT_HH
// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   20. Jul 1997
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: routines for selecting tracks by a pattern
//
// changes:
// 26 Jan 2001 (TG) creation
//
// ********************************************************************
#include <algorithm>
#include "pattern/GPatternMatch.hh"

//==============================================================================
// delete_it
//==============================================================================
// Assumption: Pointer is a pointer type!
/// delete a single pointer
template <class Pointer>
static void delete_it (Pointer obj) { delete obj; }

//==============================================================================
// dynamic_delete: empty container, call delete for each element
//==============================================================================
// Assumption: Container stores pointers to objects on heap
// Assumption: Container has T::value_type defined
/// call delete for each element of container
template <class Container>
void dynamic_delete( Container& rhs) {
  typedef typename Container::value_type Val;

  for_each(rhs.begin(), rhs.end(), delete_it<Val>);
  rhs.clear();
}


//==============================================================================
// gselect: a track selection routine
//==============================================================================
// GPatternMatch factory function!
// Assumption: Source is Container of C*! (because of GPattern::evaluate)
// Assumption: Source has T::const_iterator
// Assumption: Target is container of GPatternMatch<C,T>*!
/// A simple global track selection routine.
template <class Source, class Target, class C, class T>
bool gselect(const Source& src, Target& trg, const GPattern<C,T>& pat) {
  typedef typename Source::const_iterator Cit;

  const Cit end = src.end();
  for(Cit it = src.begin(); it != end; ++it) {
    const typename Source::value_type& track = *it;
    if( pat.evaluate(*track) ) {
      trg.push_back( new GPatternMatch<C,T>( *track, pat ) );
    }
  }

  return true;
}
#endif
