#ifndef __DECAYPATTERNMATCH_HH
#define __DECAYPATTERNMATCH_HH
// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   01. Feb 2001
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: ROOT specific class describing a decay pattern
//              match for RecoTracks and Vt++ vertices
//
// changes:
// 05 Feb 2001 (TG) creation
//
// ********************************************************************

#include <iosfwd>
#include "arte/TObject.hh"
#include "pattern/GDecayPatternMatch.hh"

namespace VERTEX { class Vertex; }
class DecayPattern;
template <class C, class T> class GDecayPatternMatch;
template <class C, class T> class GDecayPattern;
typedef GDecayPatternMatch<RecoTrack,int>  DecayPatternMatchBase;
typedef GDecayPattern<RecoTrack,int>       DPatternBase;

/** DecayPatternMatch is a specialization of the GDecayPatternMatch class for
    RecoTrack objects. DecayPatternMatch is integrated into the ROOT RTTI
    system.

    @memo ROOT class
    @author T. Glebe
 */
//==============================================================================
// DecayPatternMatch
//==============================================================================
class DecayPatternMatch : public TObject, public DecayPatternMatchBase {
public:
  /** @name --- Constructors --- */
  ///
  DecayPatternMatch();
  ///
  DecayPatternMatch(const VERTEX::Vertex& v, const DecayPattern& vpat,
		    bool cc = false);
#ifndef __CINT__
  ///
  DecayPatternMatch(const SmartPointer<VERTEX::Vertex>& v,
		    const SmartPointer<DPatternBase>& vpat,
		    bool cc = false);
#endif
  ///
  virtual ~DecayPatternMatch();

  /** @name --- Expert functions --- */
  ///
//    inline bool TrackCollision(const DecayPatternMatch& rhs) const {
//      return DecayPatternMatchBase::TrackCollision(rhs);
//    }

#ifdef USE_ROOT
  ClassDef(DecayPatternMatch,1)
#endif
}; // end of DecayPatternMatch


//==============================================================================
// operator <<
//==============================================================================
inline std::ostream& operator<<( std::ostream& os, DecayPatternMatch& a ) {
  return a.print( os );
}
#endif
