// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   20. Jul 1997
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: Pattern class for RecoTracks
//
// changes:
// 25 Jan 2001 (TG) creation
// 29 Jan 2001 (TG) moved CTOR code from Pattern.hh to here, added 
//                  set_cprop(), use_cc() code
// 31 Jan 2001 (TG) bug fix in CTOR, select() function added
// 31 Jan 2001 (TG) added operator|()
// 08 Mar 2001 (TG) CTOR added
//
// ********************************************************************

#include "pattern/Pattern.hh"
#include "pattern/PatternMatch.hh"
#include "clue/cluearte.hh" // needed by RecoTrack.hh
#include "clue/RecoTrack.hh"
//#include "pattern/RecoTrackMember.hh"
#include "pattern/DecayPattern.hh"
//#include "clue/cluearte.hh"  //because of ArteTable

//==============================================================================
// Constructors
//==============================================================================
Pattern::Pattern() : PatternBase() {}

Pattern::Pattern(const PatternBase& rhs) : 
  PatternBase(rhs) {}

Pattern::Pattern(const Pattern& rhs) : 
  PatternBase(rhs) {}

Pattern::Pattern(const ToppIf& gt) :
  PatternBase(gt) {}

Pattern::Pattern(const ToppIf& gt, const ClPropertyBase& rhs) :
  PatternBase(gt,rhs) {}

Pattern::~Pattern() {}

Pattern& Pattern::operator=(const Pattern& rhs) {
  PatternBase::operator=(rhs);
  return *this;
}


//==============================================================================
// operator|
//==============================================================================
Pattern Pattern::operator|( const Pattern& rhs ) const {
  return PatternBase::operator|(rhs);
}

//==============================================================================
// operator&
//==============================================================================
DecayPattern Pattern::operator&( const Pattern& rhs ) const {
  //  DecayPattern tmp = PatternBase::operator&(rhs);
  // something missing here?
  return DecayPattern(Any, *this, rhs);
}


//==============================================================================
// select
//==============================================================================
bool Pattern::select(const RT_t&  src, PM_l& trg) const {
  typedef RT_t::const_iterator Cit;
  
  const Cit end = src.end();
  for(Cit it = src.begin(); it != end; ++it) {
    const RT_t::value_type& track = *it;
    if( evalTrack(*track) ) {
      trg.push_back( new PatternMatch( *track, *this ) );
    }
  }
  return true;
}


#ifdef USE_ROOT
ClassImp(Pattern)
#endif
