#ifndef __PATTERN_HH
#define __PATTERN_HH
// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   20. Jul 1997
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: Pattern class for RecoTracks
//
// changes:
// 25 Jan 2001 (TG) creation
// 29 Jan 2001 (TG) added cprop, useCC members, set_cprop(), use_cc()
// 31 Jan 2001 (TG) added operator|(), select()
// 01 Feb 2001 (TG) moved cprop, useCC, set_cprop(), use_cc() to GProperty
// 08 Mar 2001 (TG) CTOR added
//
// ********************************************************************

#include <iosfwd>
#include "arte/TObject.hh"
#include "pattern/GPattern.hh"

class RecoTrack;
class DecayPattern;
template <class T> class ArteTable;
typedef GPattern<RecoTrack,int>   PatternBase;
typedef ClProperty<RecoTrack,int> ClPropertyBase;
class PatternMatch;

#include <list>
typedef list<PatternMatch*>                 PM_l;
typedef list<PatternMatch*>::iterator       PM_it;
typedef list<PatternMatch*>::const_iterator PM_cit;
typedef ArteTable<RecoTrack>                RT_t;

/** Pattern is a GPattern specialization for RecoTrack objects. Pattern
    is integrated into the ROOT RTTI system.

    @memo ROOT class
    @author T. Glebe
*/
//==============================================================================
// Pattern
//==============================================================================
class Pattern : public TObject, public PatternBase {
public:
  /** @name --- Constructors --- */
  ///
  Pattern();
  ///
  Pattern(const PatternBase& rhs);
  ///
  Pattern(const Pattern& rhs);
  ///
  Pattern(const ToppIf& gt);
  ///
  Pattern(const ToppIf& gt, const ClPropertyBase& rhs);
  ///
  virtual ~Pattern();
  ///
  Pattern& operator=(const Pattern& rhs);

  /** @name --- Operators --- */
  /// OR of two patterns
  Pattern operator|( const Pattern& rhs ) const;
  /// AND of two patterns
  DecayPattern operator&( const Pattern& rhs ) const;

  /** @name --- Expert functions --- */
  /// select patterns
  bool select(const RT_t& src, PM_l& trg) const;
  /// does a pattern match exist?
  inline bool exist(const RT_t& src) const {
    return PatternBase::exist(src);
  }

#ifdef USE_ROOT
  ClassDef(Pattern,1)
#endif
}; // end of class Pattern


//==============================================================================
// operator <<
//==============================================================================
inline std::ostream& operator<<( std::ostream& os, const Pattern& a ) {
  return a.print( os );
}

#endif
