#ifndef __PATTERNMATCH_HH
#define __PATTERNMATCH_HH
// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   20. Jul 1997
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: A new PatternMatch<RecoTrack,int> class
//
// changes:
// 23 Jan 2001 (TG) creation
// 28 Feb 2001 (TG) constructor added
//
// ********************************************************************

#include <iosfwd>
#include "arte/TObject.hh"
#include "pattern/GPatternMatch.hh"

class RecoTrack;
class Pattern;
typedef GPatternMatch<RecoTrack,int> PatternMatchBase;

/** PatternMatch is a specialization of the GPatternMatch class for RecoTrack
    objects. PatternMatch is integrated into the ROOT RTTI system.

    @memo ROOT class
    @author T. Glebe
*/
//==============================================================================
// PatternMatch
//==============================================================================
class PatternMatch : public TObject, public PatternMatchBase {
public:
  /** @name --- Constructors --- */
  ///
  PatternMatch() : PatternMatchBase() {}
  ///
  PatternMatch(const PatternMatchBase& rhs) : PatternMatchBase(rhs) {}
  ///
  PatternMatch(const RecoTrack& tr, const Pattern& pat);
  ///
  virtual ~PatternMatch() {}

#ifdef USE_ROOT
  ClassDef(PatternMatch,1)
#endif
}; // end of PatternMatch

//==============================================================================
// operator <<
//==============================================================================
inline std::ostream& operator<<( std::ostream& os, const PatternMatch& a ) {
  return a.print( os );
}

#endif
